@@ -23,7 +23,7 @@ import asyncio
 import functools
 import uuid
 
-from datetime import datetime, timedelta
+from datetime import datetime
 from enum import Enum
 
 import aioxmpp.callbacks
@@ -37,9 +37,11 @@ import aioxmpp.im.conversation
 import aioxmpp.im.dispatcher
 import aioxmpp.im.p2p
 import aioxmpp.im.service
+import aioxmpp.utils
 
 from aioxmpp.utils import namespaces
 
+from . import self_ping
 from . import xso as muc_xso
 
 
@@ -458,8 +460,9 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
     .. signal:: on_enter(**kwargs)
 
         Emits when the initial room :class:`~.Presence` stanza for the
-        local JID is received. This means that the join to the room is complete;
-        the message history and subject are not transferred yet though.
+        local JID is received. This means that the join to the room is
+        complete; the message history and subject are not transferred yet
+        though.
 
         .. seealso::
 
@@ -709,6 +712,58 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
 
             The `muc_status_codes` argument was added.
 
+    .. signal:: on_muc_stale(**kwargs)
+
+        Emits when the :attr:`muc_hard_timeout` expires.
+
+        This signal is emitted only up to once for each pause in data
+        reception. As long as data is received often enough, the timeout will
+        not trigger. When the timeout triggers due to silence, the signal is
+        emitted once, and not again until after data has been received for the
+        next time.
+
+        This signal is only informational. It does not imply that the MUC is
+        unreachable or that the local occupant has been removed from the MUC,
+        but it is very likely that no messages can currently be sent or
+        received.
+
+        It is not clear whether messages are being lost.
+
+        A prominent example on when this condition can occur is highlighted in
+        the specification for the feature this is built on (:xep:`0410`).
+        Often, the MUC service is on a remote domain, which means that there
+        are at least two network connections involved, sometimes three (c2s,
+        s2s, and from the remote server to the MUC component).
+
+        When the s2s connection (for example) fails in certain ways, it is
+        possible that no error replies are generated by any party; stanzas are
+        essentially blackholed. When the network connection resumes, it depends
+        on the exact failure mode whether the occupant is still in the room and
+        which messages (if any) which were sent in the meantime will have been
+        delivered to any participant.
+
+        After :meth:`on_muc_stale` emits, exactly one of the following will
+        happen, given infinite time:
+
+        - :meth:`on_muc_fresh` is emitted, which means that connectivity to
+          the MUC has been re-confirmed.
+
+        - :meth:`on_muc_suspend` is emitted, which means that the local client
+          has disconnected (but autorejoin is enabled).
+
+        - :meth:`on_exit` is emitted, which means that the client has been
+          removed from the MUC or the local client has disconnected (and
+          autorejoin is disabled).
+
+        The aliveness checks are only enabled after presence synchronisation
+        has begun.
+
+    .. signal:: on_muc_fresh(**kwargs)
+
+        Emits after :meth:`on_muc_stale` when connectivity is re-confirmed.
+
+        See :meth:`on_muc_stale` for details.
+
     The following signals inform users about state changes related to **other**
     occupants in the chat room. Note that different events may fire for the
     same presence stanza. A common example is a ban, which triggers
@@ -731,14 +786,14 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
         :type status_codes: :class:`~.abc.Set` of :class:`int` or
             :class:`~.StatusCode`
 
-        `occupant` is the :class:`Occupant` instance tracking the occupant whose
-        affiliation changed.
+        `occupant` is the :class:`Occupant` instance tracking the occupant
+        whose affiliation changed.
 
         .. versionchanged:: 0.10
 
             The `status_codes` argument was added.
 
-    .. signal:: on_muc_role_changed(member, *, actor=None, reason=None, status_codes=set(), , **kwargs)
+    .. signal:: on_muc_role_changed(member, *, actor=None, reason=None, status_codes=set(), **kwargs)
 
         Emits when the role of an `occupant` in the room changes.
 
@@ -753,18 +808,74 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
         :type status_codes: :class:`~.abc.Set` of :class:`int` or
             :class:`~.StatusCode`
 
-        `occupant` is the :class:`Occupant` instance tracking the occupant whose
-        role changed.
+        `occupant` is the :class:`Occupant` instance tracking the occupant
+        whose role changed.
 
         .. versionchanged:: 0.10
 
             The `status_codes` argument was added.
 
-    """
+    Timeout control:
+
+    .. seealso::
+
+        :ref:`api-aioxmpp.muc-self-ping-logic`
+
+    .. attribute:: muc_soft_timeout
+
+        The soft timeout of the MUC aliveness timeout logic as
+        :class:`datetime.timedelta`.
+
+        .. versionadded:: 0.11
+
+    .. attribute:: muc_hard_timeout
+
+        The hard timeout of the MUC aliveness timeout logic as
+        :class:`datetime.timedelta`.
+
+        .. versionadded:: 0.11
+
+    .. attribute:: muc_ping_interval
+
+        The interval at which pings are sent after the soft timeout expires as
+        :class:`datetime.timedelta`.
+
+        .. warning::
+
+            Please see the notes on :attr:`muc_ping_timeout`
+            when changing the value of :attr:`muc_ping_timeout` or
+            :attr:`muc_ping_interval`.
+
+        .. versionadded:: 0.11
+
+    .. attribute:: muc_ping_timeout
+
+        The maximum time to wait for a ping reply for each individual ping as
+        :class:`datetime.timedelta`.
+
+        .. warning::
+
+            Pings are continued to be sent even when other pings are already
+            in-flight. This means that up to
+            ``math.ceil(muc_ping_timeout / muc_ping_interval)`` pings are
+            in-flight at the same time. Each ping which is in-flight
+            unfortunately requires a small amount of memory and an entry in a
+            map which associates the stanza ID with the handler/future for the
+            reply.
+
+        .. versionadded:: 0.11
+
+    .. seealso::
+
+        :ref:`api-aioxmpp.muc-self-ping-logic`
+
+    """  # NOQA: E501
     # this occupant state events
     on_muc_suspend = aioxmpp.callbacks.Signal()
     on_muc_resume = aioxmpp.callbacks.Signal()
     on_muc_enter = aioxmpp.callbacks.Signal()
+    on_muc_stale = aioxmpp.callbacks.Signal()
+    on_muc_fresh = aioxmpp.callbacks.Signal()
 
     # other occupant state events
     on_muc_affiliation_changed = aioxmpp.callbacks.Signal()
@@ -790,6 +901,14 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
         self._service_member = ServiceMember(mucjid)
         self.muc_autorejoin = False
         self.muc_password = None
+        self._monitor = self_ping.MUCMonitor(
+            mucjid,
+            service.client,
+            self._monitor_stale,
+            self._monitor_fresh,
+            self._monitor_exited,
+            self._service.logger.getChild("MUCMonitor"),
+        )
 
     @property
     def service(self):
@@ -881,7 +1000,8 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
     @property
     def service_member(self):
         """
-        A :class:`ServiceMember` object which represents the MUC service itself.
+        A :class:`ServiceMember` object which represents the MUC service
+        itself.
 
         This is used when messages from the MUC service are received.
 
@@ -915,11 +1035,32 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
             aioxmpp.im.conversation.ConversationFeature.INVITE_DIRECT,
         }
 
+    muc_soft_timeout = aioxmpp.utils.proxy_property(
+        "_monitor",
+        "soft_timeout",
+    )
+
+    muc_hard_timeout = aioxmpp.utils.proxy_property(
+        "_monitor",
+        "hard_timeout",
+    )
+
+    muc_ping_timeout = aioxmpp.utils.proxy_property(
+        "_monitor",
+        "ping_timeout",
+    )
+
+    muc_ping_interval = aioxmpp.utils.proxy_property(
+        "_monitor",
+        "ping_interval",
+    )
+
     def _enter_active_state(self):
         self._state = RoomState.ACTIVE
         self._history_replay_occupants.clear()
 
     def _suspend(self):
+        self._monitor.disable()
         self.on_muc_suspend()
         self._active = False
         self._state = RoomState.DISCONNECTED
@@ -928,6 +1069,7 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
     def _disconnect(self):
         if not self._joined:
             return
+        self._monitor.disable()
         self.on_exit(
             muc_leave_mode=LeaveMode.DISCONNECTED
         )
@@ -943,6 +1085,18 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
         self._state = RoomState.JOIN_PRESENCE
         self.on_muc_resume()
 
+    def _monitor_stale(self):
+        self.on_muc_stale()
+
+    def _monitor_fresh(self):
+        self.on_muc_fresh()
+
+    def _monitor_exited(self):
+        if self.muc_autorejoin:
+            self._service._cycle(self)
+        else:
+            self._disconnect()
+
     def _match_tracker(self, message):
         try:
             tracker = self._tracking_by_id[message.id_]
@@ -1004,6 +1158,9 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
                                    self._mucjid,
                                    message)
 
+        self._monitor.enable()
+        self._monitor.reset()
+
         if self._state == RoomState.HISTORY and not message.xep0203_delay:
             # WORKAROUND: prosody#1053; AFFECTS: <= 0.9.12, <= 0.10
             self._service.logger.debug(
@@ -1175,6 +1332,8 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
     def _handle_self_presence(self, stanza):
         info = Occupant.from_presence(stanza, True)
 
+        self._monitor.ping_address = stanza.from_
+
         if not self._active:
             if stanza.type_ == aioxmpp.structs.PresenceType.UNAVAILABLE:
                 self._service.logger.debug(
@@ -1218,6 +1377,7 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
                                        self._mucjid,
                                        reason)
             existing.update(info)
+            self._monitor.disable()
             self.on_exit(muc_leave_mode=mode,
                          muc_actor=actor,
                          muc_reason=reason,
@@ -1230,6 +1390,9 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
                                    self._mucjid,
                                    stanza)
 
+        self._monitor.enable()
+        self._monitor.reset()
+
         if stanza.from_.is_bare:
             self._service.logger.debug(
                 "received muc user presence from bare JID %s. ignoring.",
@@ -1282,11 +1445,13 @@ class Room(aioxmpp.im.conversation.AbstractConversation):
         elif mode == _OccupantDiffClass.LEFT:
             mode, actor, reason = data
             existing.update(info)
-            self.on_leave(existing,
-                          muc_leave_mode=mode,
-                          muc_actor=actor,
-                          muc_reason=reason,
-                          muc_status_codes=stanza.xep0045_muc_user.status_codes)
+            self.on_leave(
+                existing,
+                muc_leave_mode=mode,
+                muc_actor=actor,
+                muc_reason=reason,
+                muc_status_codes=stanza.xep0045_muc_user.status_codes
+            )
             del self._occupant_info[existing.conversation_jid]
 
     def _handle_role_request(self, form):
@@ -1802,7 +1967,7 @@ class MUCClient(aioxmpp.im.conversation.AbstractConversationService,
         This class now conforms to the :class:`~.AbstractConversationService`
         interface.
 
-    """
+    """  # NOQA: E501
 
     ORDER_AFTER = [
         aioxmpp.im.dispatcher.IMDispatcher,
@@ -1896,7 +2061,7 @@ class MUCClient(aioxmpp.im.conversation.AbstractConversationService,
     def _pending_join_done(self, mucjid, room, fut):
         try:
             fut.result()
-        except Exception as exc:
+        except (Exception, asyncio.CancelledError) as exc:
             room.on_failure(exc)
 
         if fut.cancelled():
@@ -2051,6 +2216,36 @@ class MUCClient(aioxmpp.im.conversation.AbstractConversationService,
             if not fut.done():
                 fut.set_result(None)
 
+    def _cycle(self, room: Room):
+        try:
+            room, fut, nick, history = self._pending_mucs[room.jid]
+        except KeyError:
+            # the muc is already joined
+            nick = room.me.nick
+            # we do not request history for cycle operations; there is no way
+            # to determine the right amount. this could be changed in the
+            # future.
+            history = muc_xso.History()
+            history.maxchars = 0
+            history.maxstanzas = 0
+
+        unjoin = aioxmpp.stanza.Presence(
+            type_=aioxmpp.structs.PresenceType.UNAVAILABLE,
+            to=room.jid.replace(resource=nick),
+        )
+        unjoin.xep0045_muc = muc_xso.GenericExt()
+
+        self.client.enqueue(unjoin)
+        room._suspend()
+        room._resume()
+
+        self._send_join_presence(
+            room.jid,
+            history,
+            nick,
+            room.muc_password,
+        )
+
     def get_muc(self, mucjid):
         try:
             return self._joined_mucs[mucjid]
