@@ -950,6 +950,19 @@ class TestConnectionLocation(unittest.TestCase):
             t.parse("[fe80::]:5222")
         )
 
+    def test_parse_ipv6_without_port_number(self):
+        t = xso.ConnectionLocation()
+        self.assertEqual(
+            (ipaddress.IPv6Address("fe80::"), 5222),
+            t.parse("[fe80::]")
+        )
+
+    def test_invalid_ipv6(self):
+        t = xso.ConnectionLocation()
+        with self.assertRaises(ValueError):
+            t.parse("fe80:::5222")
+
+
     def test_reject_non_integer_port_number(self):
         t = xso.ConnectionLocation()
         with self.assertRaises(ValueError):
@@ -960,11 +973,6 @@ class TestConnectionLocation(unittest.TestCase):
         with self.assertRaises(ValueError):
             t.parse("[fe80::]:1000000")
 
-    def test_reject_missing_colon(self):
-        t = xso.ConnectionLocation()
-        with self.assertRaises(ValueError):
-            t.parse("foo.bar")
-
     def test_parse_ipv4(self):
         t = xso.ConnectionLocation()
         self.assertEqual(
@@ -972,6 +980,13 @@ class TestConnectionLocation(unittest.TestCase):
             t.parse("10.0.0.1:5223")
         )
 
+    def test_parse_ipv4_without_port_number(self):
+        t = xso.ConnectionLocation()
+        self.assertEqual(
+            (ipaddress.IPv4Address("10.0.0.1"), 5222),
+            t.parse("10.0.0.1")
+        )
+
     def test_parse_hostname(self):
         t = xso.ConnectionLocation()
         self.assertEqual(
@@ -979,6 +994,13 @@ class TestConnectionLocation(unittest.TestCase):
             t.parse("foo.bar.example:5234")
         )
 
+    def test_parse_hostname_without_port_number(self):
+        t = xso.ConnectionLocation()
+        self.assertEqual(
+            ("foo.bar.example", 5222),
+            t.parse("foo.bar.example")
+        )
+
     def test_format_ipv6(self):
         t = xso.ConnectionLocation()
         self.assertEqual(
@@ -1117,6 +1139,56 @@ class TestLanguageTag(unittest.TestCase):
                 t.coerce(value)
 
 
+class TestJSON(unittest.TestCase):
+    def test_is_cdata_type(self):
+        self.assertTrue(issubclass(
+            xso.JSON,
+            xso.AbstractCDataType,
+        ))
+
+    def test_parse_loads_as_json_via_instance(self):
+        j = xso.JSON()
+
+        with contextlib.ExitStack() as stack:
+            loads = stack.enter_context(unittest.mock.patch(
+                "json.loads",
+                return_value=unittest.mock.sentinel.parsed,
+            ))
+
+            result = j.parse(
+                unittest.mock.sentinel.cdata,
+            )
+
+        loads.assert_called_once_with(unittest.mock.sentinel.cdata)
+
+        self.assertEqual(result, unittest.mock.sentinel.parsed)
+
+    def test_format_dumps_as_json_via_instance(self):
+        j = xso.JSON()
+
+        with contextlib.ExitStack() as stack:
+            dumps = stack.enter_context(unittest.mock.patch(
+                "json.dumps",
+                return_value=unittest.mock.sentinel.serialised,
+            ))
+
+            result = j.format(
+                unittest.mock.sentinel.data,
+            )
+
+        dumps.assert_called_once_with(unittest.mock.sentinel.data)
+
+        self.assertEqual(result, unittest.mock.sentinel.serialised)
+
+    def test_coerce_passes_everything_via_instance(self):
+        value = object()
+
+        self.assertIs(
+            xso.JSON().coerce(value),
+            value,
+        )
+
+
 class TestTextChildMap(unittest.TestCase):
     def test_is_element_type(self):
         self.assertTrue(issubclass(
@@ -1281,16 +1353,19 @@ class TestEnumCDataType(unittest.TestCase):
                 e.coerce(thing)
 
     def test_try_to_coerce_if_allow_coerce_is_set(self):
-        enum_class = unittest.mock.Mock()
-        enum_class.return_value = unittest.mock.sentinel.wrapped
         nested_t = xso.Integer()
         t = xso.EnumCDataType(
-            enum_class,
+            self.SomeEnum,
             nested_t,
             allow_coerce=True,
         )
 
         with contextlib.ExitStack() as stack:
+            enum_constructor = stack.enter_context(unittest.mock.patch.object(
+                type(self.SomeEnum), "__call__"
+            ))
+            enum_constructor.return_value = unittest.mock.sentinel.wrapped
+
             w = stack.enter_context(warnings.catch_warnings())
             coerce = stack.enter_context(
                 unittest.mock.patch.object(nested_t, "coerce")
@@ -1299,7 +1374,7 @@ class TestEnumCDataType(unittest.TestCase):
             coerce.return_value = unittest.mock.sentinel.coerced
             result = t.coerce(unittest.mock.sentinel.value)
 
-        enum_class.assert_called_with(
+        enum_constructor.assert_called_with(
             unittest.mock.sentinel.coerced,
         )
 
@@ -1310,15 +1385,25 @@ class TestEnumCDataType(unittest.TestCase):
 
         self.assertFalse(w)
 
+    def test_pass_Enum_values_through_coerce_if_coercion_is_allowed(self):
+        e = xso.EnumCDataType(self.SomeEnum, allow_coerce=True)
+        for enum_value in self.SomeEnum:
+            self.assertIs(enum_value, e.coerce(enum_value))
+
     def test_value_error_propagates(self):
         exc = ValueError()
 
-        enum_class = unittest.mock.Mock()
-        enum_class.side_effect = exc
-        e = xso.EnumCDataType(enum_class, xso.Integer(), allow_coerce=True)
+        with contextlib.ExitStack() as stack:
+            enum_constructor = stack.enter_context(unittest.mock.patch.object(
+                type(self.SomeEnum), "__call__"
+            ))
+            enum_constructor.side_effect = exc
+            e = xso.EnumCDataType(self.SomeEnum,
+                                  xso.Integer(),
+                                  allow_coerce=True)
 
-        with self.assertRaises(ValueError) as ctx:
-            e.coerce(1234)
+            with self.assertRaises(ValueError) as ctx:
+                e.coerce(1234)
 
         self.assertIs(ctx.exception, exc)
 
